package main

import "core:runtime";
import "core:fmt";
import "core:os";
import stbtt "vendor:stb/truetype";

import "gfx";

gfx_cx: ^gfx.cx = nil

frame_func :: proc "c" (mouse_x: int, mouse_y: int, mouse_left_down: bool, mouse_right_down: bool) {
    context = runtime.default_context();

    for y in 0..<100 {
        for x in 0..<100 {
            gfx.queue_text(gfx_cx, "Helope!", { f32(x)*100, f32(x)*24 + f32(y)*26}, 1000, 1000, { 1, 1, 1, 1 });
        }
    }
}

main :: proc() {
    gfx_cx = gfx.init_context(frame_func, 800, 600)

    font_data, success := os.read_entire_file_from_filename("./bin/JetBrainsMono-Medium.ttf");
    if !success {
        fmt.eprintln("failed to read font file");
        os.exit(1);
    }

    font: stbtt.fontinfo;
    if stbtt.InitFont(&font, raw_data(font_data), 0) == false {
        fmt.eprintln("failed to init font");
        os.exit(1);
    }

    bitmap_size: i32 = 512;
    bitmap := make([]u8, bitmap_size * bitmap_size);

    rasterized_font_height: i32 = 24;

    ascent, descent, line_gap: i32;
    scale := stbtt.ScaleForPixelHeight(&font, f32(rasterized_font_height * 2));
    stbtt.GetFontVMetrics(&font, &ascent, &descent, &line_gap);

    gfx.add_glyph(gfx_cx, gfx.GpuGlyph {
        size = {f32(rasterized_font_height/4), 1},
        y_offset = f32(-rasterized_font_height),
    });

    x := rasterized_font_height / 4;
    y: i32 = 0;
    for i in 33..<33+96 {
        width, height, xoff, yoff: i32;
        glyph_bitmap := stbtt.GetCodepointBitmap(
            &font, scale, scale, rune(i), &width, &height, &xoff, &yoff);

        if (x + width) >= bitmap_size {
            x = 0;
            y += i32(f32(ascent - descent + line_gap) * scale);
        }

        xxx: i32 = x;
        for xx in 0..<width {
            yyy: i32 = y;
            for yy in 0..<height {
                bitmap[xxx + yyy * bitmap_size] =
                    glyph_bitmap[xx + yy * width];

                yyy += 1;
            }

            xxx += 1;
        }

        gfx.add_glyph(gfx_cx,
            gfx.GpuGlyph {
                atlas_position = {f32(x), f32(y)},
                size = {f32(width), f32(height)},
                position = {f32(x), f32(y)},
                y_offset = f32(yoff),
            }
        );

        x += width;
    }
    gfx.push_texture_buffer(gfx_cx, u32(bitmap_size), u32(bitmap_size), raw_data(bitmap), u32(bitmap_size * bitmap_size));

    for(gfx.keep_running) {
        gfx.run_events(gfx_cx)
    }
}
